#include "contourleveldialog.h"
#include "hullitem.h"
#include "graleeditor.h"

ContourLevelDialog::ContourLevelDialog(QWidget *pParent, const QVector<QVector<QPointF>> &contours, HullItem *pHullItem)
	: QDialog(pParent)
{
	setupUi(this);

	m_contours = contours;
	m_pHullItem = pHullItem;
	QObject::connect(m_pContourLevelSlider, &QSlider::sliderMoved, this, &ContourLevelDialog::slotSliderMoved);
	QObject::connect(m_pContourLevelSlider, &QSlider::valueChanged, this, &ContourLevelDialog::slotSliderMoved);

	move(0, 0);

	const int maxLevel = contours.size() - 1;
	m_pContourLevelSlider->setMinimum(0);
	m_pContourLevelSlider->setMaximum(maxLevel);
	m_pContourLevelSlider->setValue(maxLevel/2);

	int lowestValue = 0;
	while (contours[lowestValue].size() == 0 && lowestValue < maxLevel)
		lowestValue++;
	int highestValue = maxLevel;
	while (contours[highestValue].size() == 0 && highestValue > 0)
		highestValue--;

	if (lowestValue > highestValue)
		m_foundContour = false;
	else
		m_foundContour = true;

	if (lowestValue < highestValue && lowestValue <= maxLevel && highestValue >= 0)
	{
		double diff = highestValue-lowestValue;
		int startValue = (int)(lowestValue + diff/5.0 + 0.5); // TODO: make this configurable
		if (startValue < lowestValue)
			startValue = lowestValue;
		if (startValue > highestValue)
			startValue = highestValue;

		m_pContourLevelSlider->setMinimum(lowestValue);
		m_pContourLevelSlider->setMaximum(highestValue);
		m_pContourLevelSlider->setValue(startValue);
	}

	slotSliderMoved(m_pContourLevelSlider->value());
}

ContourLevelDialog::~ContourLevelDialog()
{
}

void ContourLevelDialog::slotSliderMoved(int pos)
{
	if (!(pos >= 0 && pos < m_contours.size()))
		return;

	auto &contour = m_contours[pos];
	bool first = true;
	for (auto p : contour)
	{
		if (first)
			m_pHullItem->startHull(p);
		else
			m_pHullItem->addHullPoint(p);
		first = false;
	}

	// Close the contour
	if (contour.size() > 0)
		m_pHullItem->addHullPoint(contour[0]);
}
