/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef EDITORSCENE_H

#define EDITORSCENE_H

#include <QGraphicsScene>
#include <QList>
#include <errut/errorbase.h>
#include <serut/serializationinterface.h>
#include "layeritem.h"

class EditorScene : public QGraphicsScene, public errut::ErrorBase
{
	Q_OBJECT
public:
	EditorScene();
	~EditorScene();

	void clear();
	int addLayer(LayerItem::LayerType layerType);
	LayerItem *getLayerItem(int i)						{ if (i >= m_layers.size() || i < 0) return 0; return m_layers.at(i); }
	int getNumberOfLayers() const						{ return m_layers.size(); }
	void setCurrentLayer(int i);
	int getCurrentLayer() const						{ return m_currentLayer; }
	
	void decreaseLayerIndex(int l);
	void increaseLayerIndex(int l);

	void deleteLayer(int l);

	bool load(const QString &fname);
	bool save(const QString &fname) const;
	bool load(serut::SerializationInterface &si);
	bool save(serut::SerializationInterface &si) const;
	bool importImagesData(const QString &fname, QList<LayerItem *> &newLayers);
	bool exportImagesData(const QString &fname, bool triangles, bool intensities, bool groups, bool timeDelays);

	void enableMoveAndSelect(bool move); // for current layer
	void disableMoveAndSelect(); // for current layer

	void setCaching(bool flags);

	double getDefaultPointScaleFactor() const							{ return m_defaultPointScaleFactor; }
	double getDefaultMatchPointScaleFactor() const							{ return m_defaultMatchPointScaleFactor; }
	void setAllPointSizes(double s);
	void setAllMatchPointSizes(double s);

	bool isCoordinateItemShown() const;
	void setCoordinateItem(bool show);
public slots:
	void toggleCoordinateItem();
	void setDefaultPointScaleFactor(double v);
	void setDefaultMatchPointScaleFactor(double v);
private:
	void mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void objectFinder(QPointF point, LayerItem *pLayer);
	void findObjectAtCenter(QRectF region, const QPixmap &pixmap, LayerItem *pLayer);
	void updateZValues();

	QList<LayerItem *> m_layers;
	QGraphicsItem *m_pCoordinateItem;
	int m_currentLayer;
	double m_defaultPointScaleFactor;
	double m_defaultMatchPointScaleFactor;
};

#endif // EDITORSCENE_H
