/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "gridinputdialog.h"
#include <QIntValidator>
#include <QDoubleValidator>
#include <QSettings>

bool GridInputDialog::s_first = true;
int GridInputDialog::s_lastXPoints = 64;
int GridInputDialog::s_lastYPoints = 64;
double GridInputDialog::s_lastXCenter = 0.0;
double GridInputDialog::s_lastYCenter = 0.0;
double GridInputDialog::s_lastWidth = 100.0;
double GridInputDialog::s_lastHeight = 100.0;
bool GridInputDialog::s_lastTriangles = true;
bool GridInputDialog::s_lastExclude = false;

static char kNumXPoints[] = "gridInput_numXPoints";
static char kNumYPoints[] = "gridInput_numYPoints";
static char kXCenter[] = "gridInput_xCenter";
static char kYCenter[] = "gridInput_yCenter";
static char kWidth[] = "gridInput_width";
static char kHeight[] = "gridInput_height";
static char kTriangles[] = "gridInput_triangles";
static char kExclude[] = "gridInput_exclude";

GridInputDialog::GridInputDialog(QWidget *pParent) : QDialog(pParent)
{
	setupUi(this);

	if (s_first)
	{
		s_first = false;

		QSettings settings;

		s_lastXPoints = settings.value(kNumXPoints, s_lastXPoints).toInt();
		s_lastYPoints = settings.value(kNumYPoints, s_lastYPoints).toInt();
		s_lastXCenter = settings.value(kXCenter, s_lastXCenter).toDouble();
		s_lastYCenter = settings.value(kYCenter, s_lastYCenter).toDouble();
		s_lastWidth = settings.value(kWidth, s_lastWidth).toDouble();
		s_lastHeight = settings.value(kHeight, s_lastHeight).toDouble();
		s_lastTriangles = settings.value(kTriangles, s_lastTriangles).toBool();
		s_lastExclude = settings.value(kExclude, s_lastExclude).toBool();
	}

	m_editXPoints->setValue(s_lastXPoints);
	m_editYPoints->setValue(s_lastYPoints);
	m_editXCenter->setValue(s_lastXCenter);
	m_editYCenter->setValue(s_lastYCenter);
	m_editWidth->setValue(s_lastWidth);
	m_editHeight->setValue(s_lastHeight);
	m_checkAddTriangles->setChecked(s_lastTriangles);
	m_pCheckExclude->setChecked(s_lastExclude);
}

GridInputDialog::~GridInputDialog()
{
}

int GridInputDialog::exec()
{
	int status = QDialog::exec();
	if (status == QDialog::Accepted)
	{
		s_lastXPoints = getXPoints();
		s_lastYPoints = getYPoints();
		s_lastXCenter = getXCenter();
		s_lastYCenter = getYCenter();
		s_lastWidth = getWidth();
		s_lastHeight = getHeight();
		s_lastTriangles = getAddTriangles();
		s_lastExclude = getExcludeAreas();

		QSettings settings;
		settings.setValue(kNumXPoints, s_lastXPoints);
		settings.setValue(kNumYPoints, s_lastYPoints);
		settings.setValue(kXCenter, s_lastXCenter);
		settings.setValue(kYCenter, s_lastYCenter);
		settings.setValue(kWidth, s_lastWidth);
		settings.setValue(kHeight, s_lastHeight);
		settings.setValue(kTriangles, s_lastTriangles);
		settings.setValue(kExclude, s_lastExclude);
	}
	return status;
}

int GridInputDialog::getXPoints() const
{
	return m_editXPoints->value();
}

int GridInputDialog::getYPoints() const
{
	return m_editYPoints->value();
}

double GridInputDialog::getXCenter() const
{
	return m_editXCenter->value();
}

double GridInputDialog::getYCenter() const
{
	return m_editYCenter->value();
}

double GridInputDialog::getWidth() const
{
	return m_editWidth->value();
}

double GridInputDialog::getHeight() const
{
	return m_editHeight->value();
}

bool GridInputDialog::getAddTriangles() const
{
	if (m_checkAddTriangles->checkState() == Qt::Checked)
		return true;
	return false;
}

bool GridInputDialog::getExcludeAreas() const
{
	if (m_pCheckExclude->checkState() == Qt::Checked)
		return true;
	return false;
}

