/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "hullitem.h"
#include <QPainter>

#include <iostream>

using namespace std;

HullItem::HullItem(QGraphicsItem * pParent) : QGraphicsItem(pParent)
{
}

HullItem::~HullItem()
{
}

void HullItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	QPen pen(Qt::green);
	pen.setWidth(0.0);
	painter->setPen(pen);

	if (m_hullPoints.size() == 1)
	{
			painter->drawLine(m_hullPoints[0], m_hullPoints[0]);
	}
	else
	{
		for (int i = 1 ; i < m_hullPoints.size() ; i++)
			painter->drawLine(m_hullPoints[i-1], m_hullPoints[i]);
	}
}

QRectF HullItem::boundingRect() const
{
	return m_boundingBox;
}

void HullItem::startHull(QPointF p)
{
	m_hullPoints.clear();
	m_hullPoints.push_back(p);
	m_boundingBox = QRectF(p.x(), p.y(), 0, 0);

	prepareGeometryChange();
}

void HullItem::addHullPoint(QPointF p)
{
	m_hullPoints.push_back(p);

	if (p.x() < m_boundingBox.left())
		m_boundingBox.setLeft(p.x());
	if (p.x() > m_boundingBox.right())
		m_boundingBox.setRight(p.x());
	if (p.y() < m_boundingBox.top())
		m_boundingBox.setTop(p.y());
	if (p.y() > m_boundingBox.bottom())
		m_boundingBox.setBottom(p.y());

	prepareGeometryChange();
}

void HullItem::endHull()
{
	m_hullPoints.clear();

	prepareGeometryChange();
}

