/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef LAYERITEM_H

#define LAYERITEM_H

#include <QGraphicsItem>
#include <QObject>

class PointItem;
class TriangleItem;
class PointMatchItem;
class HullItem;

class LayerItem : public QObject, public QGraphicsItem
{
	Q_OBJECT
public:
	enum LayerType { Image, Points };

	LayerItem(LayerType layerType);
	~LayerItem();

	LayerType getLayerType() const								{ return m_layerType; }

	void addPoint(QPointF pos);
	void setPointScaleFactor(double factor);
	double getPointScaleFactor() const							{ return m_pointScaleFactor; }

	void disable();
	void enable();

	bool isLayerEnabled() const								{ return m_enabled; }

	QList<PointItem *> getSelectedPoints();
	QList<TriangleItem *> getSelectedTriangles();
	QList<PointMatchItem *> getSelectedMatchPoints();
	QList<PointItem *> getAllPoints();
	QList<TriangleItem *> getAllTriangles();
	QList<PointMatchItem *> getAllMatchPoints();
	void deleteTriangles();

	int getNextGroupNumber() const								{ return m_nextGroupNumber; }
	void setGroupNumber(int n);

	void startHull(QPointF p);
	void endHull(bool cancel = false);
	void addHullPoint(QPointF p);
	bool isDrawingHull() const									{ return m_hullDrawing; }
signals:
	void signalGroupNumberChanged();
private:
	QRectF boundingRect() const;
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

	LayerType m_layerType;
	bool m_enabled;
	double m_pointScaleFactor;
	int m_nextGroupNumber;

	bool m_hullDrawing;
	HullItem *m_pHullItem;
};

#endif // LAYERITEM_H

