#include "hullprocessor.h"
#include <iostream>
#include <assert.h>

using namespace std;
using namespace grale;

double hullPoints[][2] = {
    { 27.5329, 8.93831 },
    { 27.5688, 8.93831 },
    { 27.6047, 8.93831 },
    { 27.6047, 8.90241 },
    { 27.6406, 8.90241 },
    { 27.6765, 8.90241 },
    { 27.6765, 8.86652 },
    { 27.7123, 8.86652 },
    { 27.7482, 8.83062 },
    { 27.7841, 8.79472 },
    { 27.82, 8.75882 },
    { 27.8559, 8.75882 },
    { 27.8559, 8.72293 },
    { 27.8918, 8.72293 },
    { 27.8918, 8.68703 },
    { 27.9277, 8.68703 },
    { 27.9277, 8.65113 },
    { 27.9636, 8.65113 },
    { 27.9636, 8.61524 },
    { 27.9636, 8.57934 },
    { 27.9636, 8.54344 },
    { 27.9995, 8.54344 },
    { 27.9995, 8.50755 },
    { 27.9995, 8.47165 },
    { 27.9995, 8.43575 },
    { 27.9995, 8.39986 },
    { 27.9995, 8.36396 },
    { 27.9636, 8.36396 },
    { 27.9277, 8.32806 },
    { 27.8918, 8.32806 },
    { 27.8559, 8.29217 },
    { 27.82, 8.29217 },
    { 27.7841, 8.29217 },
    { 27.7482, 8.29217 },
    { 27.7123, 8.25627 },
    { 27.6765, 8.25627 },
    { 27.6765, 8.29217 },
    { 27.6765, 8.32806 },
    { 27.6406, 8.32806 },
    { 27.6406, 8.36396 },
    { 27.6047, 8.36396 },
    { 27.6047, 8.39986 },
    { 27.5688, 8.39986 },
    { 27.5688, 8.43575 },
    { 27.5329, 8.43575 },
    { 27.5329, 8.47165 },
    { 27.5329, 8.50755 },
    { 27.497, 8.50755 },
    { 27.497, 8.54344 },
    { 27.497, 8.57934 },
    { 27.4611, 8.61524 },
    { 27.4611, 8.65113 },
    { 27.4611, 8.68703 },
    { 27.4611, 8.72293 },
    { 27.4252, 8.72293 },
    { 27.4252, 8.75882 },
    { 27.4252, 8.79472 },
    { 27.4252, 8.83062 },
    { 27.4611, 8.86652 },
    { 27.497, 8.86652 },
    { 27.5329, 8.90241 },
    { 27.5688, 8.90241 },
};

void printPoints(const QVector<QPointF> &pts, bool close = false)
{
	for (const QPointF &p : pts)
		cout << "    " << p.x() << " " << p.y() << endl;
	if (close && pts.size() > 0)
		cout << "    " << pts[0].x() << " " << pts[0].y() << endl;
}

void printTriangulation(const QVector<TriangleIndices> &triangleIndices, const QVector<QPointF> &pts)
{
	for (const TriangleIndices &t : triangleIndices)
	{
		QPointF p0 = pts[t.getIndex(0)];
		QPointF p1 = pts[t.getIndex(1)];
		QPointF p2 = pts[t.getIndex(2)];

		cout << p0.x() << " " << p0.y() << endl;
		cout << p1.x() << " " << p1.y() << endl;
		cout << p2.x() << " " << p2.y() << endl;
		cout << p0.x() << " " << p0.y() << endl;
		cout << endl;
	}
}

int main(void)
{
	QVector<QPointF> hull;
	const int numPoints = sizeof(hullPoints)/(2*sizeof(double));

	for (int i = 0 ; i < numPoints ; i++)
		hull.push_back(QPointF(hullPoints[i][0], hullPoints[i][1]));

	HullProcessor hp;

	if (!hp.createImgDataPointsFromHull(hull))
	{
		cerr << "Unable to process hull into image data points: " << hp.getErrorString() << endl;
		return -1;
	}

	auto imgDataPoints = hp.getImageDataPoints();

	cout << "# Hull:" << endl;
	printPoints(hull, true);
	cout << endl;
	cout << endl;

	cout << "# Image data points: " << endl;
	printPoints(imgDataPoints);
	cout << endl;
	cout << endl;

	cout << "# Triangulation: " << endl;
	auto triangles = hp.getTriangulation();
	printTriangulation(triangles, imgDataPoints);

	return 0;
}
