/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "pointmatchitem.h"
#include "graleeditor.h"
#include "layeritem.h"
#include <QPainter>
#include <QGraphicsTextItem>
#include <QColor>

PointMatchItem::PointMatchItem(QGraphicsItem *pParent) : QGraphicsItem(pParent)
{
	m_pGroupTextItem = 0;
	m_group = -1;

	if (pParent)
	{
		QTransform t = pParent->transform();
		QTransform t2(t.m11(), t.m12(), t.m21(), t.m22(), 0, 0);
		QTransform inverted = t2.inverted();
		
		setTransform(inverted);
		setTransform(QTransform(-1.0,0,0,-1.0,0,0), true);
	}
}

PointMatchItem::~PointMatchItem()
{
}

void PointMatchItem::setScaleFactor(double scale)
{
	QGraphicsItem *pParent = parentItem();

	if (pParent)
	{
		QTransform t = pParent->transform();
		QTransform t2(t.m11(), t.m12(), t.m21(), t.m22(), 0, 0);
		QTransform inverted = t2.inverted();
		
		setTransform(inverted);
		setTransform(QTransform(-scale,0,0,-scale,0,0), true);
	}
	else
		setTransform(QTransform(-scale, 0, 0, -scale, 0, 0), false);
}

void PointMatchItem::setGroupNumber(int id)
{
	m_group = id;

	if (id >= 0)
	{
		QString idStr = QString::number(id, 10);

		if (m_pGroupTextItem == 0)
		{
			m_pGroupTextItem = new QGraphicsTextItem(this);
			m_pGroupTextItem->setDefaultTextColor(QColor(0, 255, 255));
			m_pGroupTextItem->setZValue(0.1);
		}
		m_pGroupTextItem->setPlainText(idStr);

		QRectF r = m_pGroupTextItem->boundingRect();

		m_pGroupTextItem->setPos(-r.width()/2.0, 0);
	}
	else
	{
		if (m_pGroupTextItem != 0)
		{
			delete m_pGroupTextItem;
			m_pGroupTextItem = 0;
		}
	}

	prepareGeometryChange();
}

QRectF PointMatchItem::boundingRect() const
{
	QRectF rect = QRectF(-13,-13,26,26);
	return rect.united(childrenBoundingRect());
}

void PointMatchItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	if (isSelected())
		painter->setPen(QPen(Qt::blue, 2));
	else
		painter->setPen(QPen(Qt::red, 2));
	painter->drawLine(QPointF(-10,-10),QPointF(10,10));
	painter->drawLine(QPointF(-10,10),QPointF(10,-10));
}

void PointMatchItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
	QGraphicsItem::mousePressEvent(event);
}

void PointMatchItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
	GraleEditor::Tool tool = GraleEditor::getInstance()->getCurrentTool();

	if (tool == GraleEditor::GroupNumberTool)
	{
		LayerItem *pLayer = (LayerItem *)(parentItem()->parentItem());
		int number = pLayer->getNextGroupNumber();

		setGroupNumber(number);

		if (number >= 0)
			pLayer->setGroupNumber(number+1);
	}

	update();
	QGraphicsItem::mouseReleaseEvent(event);
}

