/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "pointslayertreeitem.h"
#include "layeritem.h"
#include <stdio.h>

PointsLayerTreeItem::PointsLayerTreeItem(LayerItem *pLayers, QTreeWidget *pLayersWidget) : LayerTreeItem(pLayers, pLayersWidget)
{
	setText(0, "Points layer");
	//setFirstColumnSpanned(true); // Can't do this - 'ACTIVE' won't show then

	TreeItemBase *pScaleItem = new TreeItemBase(TreeItemBase::PointScale, this);
	pScaleItem->setFlags(Qt::ItemIsEditable|Qt::ItemIsEnabled);

	TreeItemBase *pGroupItem = new TreeItemBase(TreeItemBase::GroupNumber, this);
	pGroupItem->setFlags(Qt::ItemIsEditable|Qt::ItemIsEnabled);

	updateProperties();
}

PointsLayerTreeItem::~PointsLayerTreeItem()
{
}

void PointsLayerTreeItem::updateProperties()
{
	int numChildren = childCount();
	double scaleFactor = m_pLayer->getPointScaleFactor();
	int nextGroupNumber = m_pLayer->getNextGroupNumber();

	for (int i = 0 ; i < numChildren ; i++)
	{
		char str[1024];
		QString label;

		TreeItemBase *pProp = (TreeItemBase *)child(i);

		if (pProp->getType() == TreeItemBase::PointScale)
		{
			label = "Point scale factor";
			sprintf(str, "%10.10f", (double)scaleFactor);
		}
		else if (pProp->getType() == TreeItemBase::GroupNumber)
		{
			label = "Next group number";
			sprintf(str, "%d", nextGroupNumber);
		}

		pProp->setText(0, label);
		pProp->setText(1, str);
	}
}

