/*

  This file is a part of GRALEEDITOR, a program to help create input data
  for gravitational lens inversion using GRALE.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "triangleitem.h"
#include "pointitem.h"
#include <QPainter>

TriangleItem::TriangleItem(PointItem *pPoint1, PointItem *pPoint2, PointItem *pPoint3, QGraphicsItem *pParent) : QGraphicsItem(pParent)
{
	m_pPoints[0] = pPoint1;
	m_pPoints[1] = pPoint2;
	m_pPoints[2] = pPoint3;

	for (int i = 0 ; i < 3 ; i++)
		m_pPoints[i]->addTriangle(this);

	setZValue(-0.1);
	//setFlag(ItemIsSelectable);
}

TriangleItem::~TriangleItem()
{
	for (int i = 0 ; i < 3 ; i++)
		m_pPoints[i]->removeTriangle(this);
}

void TriangleItem::pointChanged()
{
	prepareGeometryChange();
}

QRectF TriangleItem::boundingRect() const
{
	double minX = m_pPoints[0]->pos().x();
	double maxX = m_pPoints[0]->pos().x();
	double minY = m_pPoints[0]->pos().y();
	double maxY = m_pPoints[0]->pos().y();

	for (int i = 1 ; i < 3 ; i++)
	{
		if (m_pPoints[i]->pos().x() < minX)
			minX = m_pPoints[i]->pos().x();
		if (m_pPoints[i]->pos().x() > maxX)
			maxX = m_pPoints[i]->pos().x();
		if (m_pPoints[i]->pos().y() < minY)
			minY = m_pPoints[i]->pos().y();
		if (m_pPoints[i]->pos().y() > maxY)
			maxY = m_pPoints[i]->pos().y();
	}
	return QRectF(QPointF(minX, minY), QPointF(maxX, maxY));
}

QPainterPath TriangleItem::shape() const
{
	QPainterPath path(m_pPoints[0]->pos());

	path.lineTo(m_pPoints[1]->pos());
	path.lineTo(m_pPoints[2]->pos());
	path.closeSubpath();

	return path;
}

void TriangleItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	QPointF points[3];

	for (int i = 0 ; i < 3 ; i++)
		points[i] = m_pPoints[i]->pos();

	QPen pen(Qt::black);
	pen.setWidth(0.0);

	painter->setPen(pen);
	if (isSelected())
		painter->setBrush(QBrush(QColor(192,192,192,128)));
	else
		painter->setBrush(QBrush(QColor(128,128,128,128)));
	painter->drawConvexPolygon(points, 3);
}

